@@ -1,6 +1,7 @@
 import time
-import urlparse
+import pytz
 import urllib2
+import urlparse
 import datetime
 from jenkinsapi.artifact import Artifact
 from jenkinsapi import config
@@ -31,7 +32,11 @@ class Build(JenkinsBase):
     def __str__(self):
         return self._data['fullDisplayName']
 
-    def id(self):
+    @property
+    def name(self):
+        return str(self)
+
+    def get_number(self):
         return self._data["number"]
 
     def get_status(self):
@@ -62,13 +67,14 @@ class Build(JenkinsBase):
 
     def get_artifacts( self ):
         for afinfo in self._data["artifacts"]:
-            url = "%sartifact/%s" % ( self.baseurl, afinfo["relativePath"] )
+            url = "%s/artifact/%s" % ( self.baseurl, afinfo["relativePath"] )
             af = Artifact( afinfo["fileName"], url, self )
             yield af
-            del af, url
 
     def get_artifact_dict(self):
-        return dict( (a.url[len(a.build.baseurl + "artifact/"):], a) for a in self.get_artifacts() )
+        return dict(
+            (af.filename, af) for af in self.get_artifacts()
+        )
 
     def get_upstream_job_name(self):
         """
@@ -210,6 +216,10 @@ class Build(JenkinsBase):
         self.poll()
         return self._data["building"]
 
+    def block(self):
+        while self.is_running():
+            time.sleep(1)
+
     def is_good( self ):
         """
         Return a bool, true if the build was good.
@@ -222,7 +232,7 @@ class Build(JenkinsBase):
         count = 0
         while self.is_running():
             total_wait = delay * count
-            log.info("Waited %is for %s #%s to complete" % ( total_wait, self.job.id(), self.id() ) )
+            log.info("Waited %is for %s #%s to complete" % ( total_wait, self.job.name, self.name ) )
             sleep( delay )
             count += 1
 
@@ -263,8 +273,12 @@ class Build(JenkinsBase):
         return all_actions
 
     def get_timestamp(self):
+        '''
+        Returns build timestamp in UTC
+        '''
         # Java timestamps are given in miliseconds since the epoch start!
-        return datetime.datetime(*time.localtime(self._data['timestamp']/1000.0)[:6])
+        naive_timestamp = datetime.datetime(*time.gmtime(self._data['timestamp']/1000.0)[:6])
+        return pytz.utc.localize(naive_timestamp)
 
     def stop(self):
         """
