@@ -1,19 +1,19 @@
+import json
 import logging
 import urlparse
-import urllib2
-import urllib
 import xml.etree.ElementTree as ET
 from collections import defaultdict
 from time import sleep
 from jenkinsapi.build import Build
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi import exceptions
+from jenkinsapi.mutable_jenkins_thing import MutableJenkinsThing
 
-from exceptions import NoBuildData, NotFound, NotInQueue
+from jenkinsapi.exceptions import NoBuildData, NotFound, NotInQueue, WillNotBuild
 
 log = logging.getLogger(__name__)
 
-class Job(JenkinsBase):
+class Job(JenkinsBase, MutableJenkinsThing):
     """
     Represents a jenkins job
     A job can hold N builds which are the actual execution environments
@@ -44,9 +44,6 @@ class Job(JenkinsBase):
             }
         JenkinsBase.__init__( self, url )
 
-    def id( self ):
-        return self._data["name"]
-
     def __str__(self):
         return self._data["name"]
 
@@ -67,65 +64,87 @@ class Job(JenkinsBase):
             self._element_tree = ET.fromstring(self._config)
         return self._element_tree
 
-    def get_build_triggerurl(self, token=None, params=None):
-        if token is None and params is None:
-            extra = "build"
-        elif params:
-            if token:
-                assert isinstance(token, str ), "token if provided should be a string."
-                params['token'] = token
-            extra = "buildWithParameters"
-        else:
-            assert isinstance(token, str ), "token if provided should be a string."
-            params = dict()
-            params['token'] = token
-            extra = "build"
-        buildurl = urlparse.urljoin( self.baseurl, extra )
-        return buildurl, params
+    def get_build_triggerurl(self):
+        return "%s/build" % self.baseurl
 
-    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3, invoke_block_delay=15, params=None, cause=None):
+    @staticmethod
+    def _mk_json_from_build_parameters(build_params):
+        """
+        Build parameters must be submitted in a particular format - Key-Value pairs would be
+        far too simple, no no! Watch and read on and behold!
+        """
+        assert isinstance(build_params, dict), 'Build parameters must be a dict'
+        return {'parameter':[
+            {'name':k, 'value':v} for k,v in build_params.iteritems()
+            ]}
+
+    @staticmethod
+    def mk_json_from_build_parameters(build_params):
+        to_json_structure = Job._mk_json_from_build_parameters(build_params)
+        return json.dumps(to_json_structure)
+
+    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3, invoke_block_delay=15, build_params=None, cause=None):
         assert isinstance( invoke_pre_check_delay, (int, float) )
         assert isinstance( invoke_block_delay, (int, float) )
         assert isinstance( block, bool )
         assert isinstance( skip_if_running, bool )
+
+        # Either copy the params dict or make a new one.
+        build_params = build_params and dict(build_params.items()) or {} # Via POSTed JSON
+        params = {} # Via Get string
+
         if self.is_queued():
-            log.warn( "Will not request new build because %s is already queued" % self.id() )
-            pass
+            raise WillNotBuild('%s is already queued' % repr(self))
+
         elif self.is_running():
             if skip_if_running:
-                log.warn( "Will not request new build because %s is already running" % self.id() )
-                pass
+                log.warn( "Will not request new build because %s is already running", self.name )
             else:
-                log.warn("Will re-schedule %s even though it is already running" % self.id() )
+                log.warn("Will re-schedule %s even though it is already running", self.name )
         original_build_no = self.get_last_buildnumber()
-        log.info( "Attempting to start %s on %s" % ( self.id(), repr(self.get_jenkins_obj()) ) )
-        url, params = self.get_build_triggerurl( securitytoken, params)
+        log.info( "Attempting to start %s on %s", self.name, repr(self.get_jenkins_obj()) )
+
+        url  = self.get_build_triggerurl()
+
         if cause:
-            params['cause'] = cause
-        html_result = self.hit_url(url, params)
-        assert len( html_result ) > 0
+            build_params['cause'] = cause
+
+        if securitytoken:
+            params['token'] = securitytoken
+
+        response = self.jenkins.requester.post_and_confirm_status(
+            url,
+            data={'json':self.mk_json_from_build_parameters(build_params)}, # See above - build params have to be JSON encoded & posted.
+            params=params
+        )
+
+        assert len( response.text ) > 0
         if invoke_pre_check_delay > 0:
-            log.info("Waiting for %is to allow Jenkins to catch up" % invoke_pre_check_delay )
+            log.info("Waiting for %is to allow Jenkins to catch up" , invoke_pre_check_delay )
             sleep( invoke_pre_check_delay )
         if block:
             total_wait = 0
+
             while self.is_queued():
-                log.info( "Waited %is for %s to begin..." % ( total_wait, self.id() ) )
+                log.info( "Waited %is for %s to begin...", total_wait, self.name  )
                 sleep( invoke_block_delay )
                 total_wait += invoke_block_delay
             if self.is_running():
                 running_build = self.get_last_build()
                 running_build.block_until_complete( delay=invoke_pre_check_delay )
+
             assert self.get_last_buildnumber() > original_build_no, "Job does not appear to have run."
         else:
             if self.is_queued():
-                log.info( "%s has been queued." % self.id() )
+                log.info( "%s has been queued.", self.name )
             elif self.is_running():
-                log.info( "%s is running." % self.id() )
+                log.info( "%s is running.", self.name )
             elif original_build_no < self.get_last_buildnumber():
-                log.info( "%s has completed." % self.id() )
+                log.info( "%s has completed.", self.name )
+            elif not response.ok :
+                response.raise_for_status()
             else:
-                raise AssertionError("The job did not schedule.")
+                raise AssertionError("The job did not schedule. REASON:%s" % response.reason)
 
     def _buildid_for_type(self, buildtype):
         """Gets a buildid for a given type of build"""
@@ -177,6 +196,12 @@ class Job(JenkinsBase):
         """
         return reversed( sorted( self.get_build_dict().keys() ) )
 
+    def get_next_build_number(self):
+        """
+        Return the next build number that Jenkins will assign.
+        """
+        return self._data.get('nextBuildNumber', 0)
+
     def get_last_good_build( self ):
         """
         Get the last good build
@@ -245,12 +270,13 @@ class Job(JenkinsBase):
             if build is not None:
                 return build.is_running()
         except NoBuildData:
-            log.info("No build info available for %s, assuming not running." % str(self) )
+            log.info("No build info available for %s, assuming not running.", str(self) )
         return False
 
     def get_config(self):
         '''Returns the config.xml from the job'''
-        return self.hit_url("%(baseurl)s/config.xml" % self.__dict__)
+        response = self.jenkins.requester.get_and_confirm_status("%(baseurl)s/config.xml" % self.__dict__)
+        return response.text
 
     def load_config(self):
         self._config = self.get_config()
@@ -263,12 +289,12 @@ class Job(JenkinsBase):
             raise exceptions.NotSupportSCM("SCM class \"%s\" not supported by API, job \"%s\"" % (scm_class, self.name))
         if scm == 'NullSCM':
             raise exceptions.NotConfiguredSCM("SCM does not configured, job \"%s\"" % self.name)
-        return scm 
+        return scm
 
     def get_scm_url(self):
         """
         Get list of project SCM urls
-        For some SCM's jenkins allow to configure and use number of SCM url's 
+        For some SCM's jenkins allow to configure and use number of SCM url's
         : return: list of SCM urls
         """
         element_tree = self._get_config_element_tree()
@@ -288,10 +314,10 @@ class Job(JenkinsBase):
     def modify_scm_branch(self, new_branch, old_branch=None):
         """
         Modify SCM ("Source Code Management") branch name for configured job.
-        :param new_branch : new repository branch name to set. 
-                            If job has multiple branches configured and "old_branch" 
+        :param new_branch : new repository branch name to set.
+                            If job has multiple branches configured and "old_branch"
                             not provided - method will allways modify first url.
-        :param old_branch (optional): exact value of branch name to be replaced. 
+        :param old_branch (optional): exact value of branch name to be replaced.
                             For some SCM's jenkins allow set multiple branches per job
                             this parameter intended to indicate which branch need to be modified
         """
@@ -311,8 +337,8 @@ class Job(JenkinsBase):
     def modify_scm_url(self, new_source_url, old_source_url=None):
         """
         Modify SCM ("Source Code Management") url for configured job.
-        :param new_source_url : new repository url to set. 
-                                If job has multiple repositories configured and "old_source_url" 
+        :param new_source_url : new repository url to set.
+                                If job has multiple repositories configured and "old_source_url"
                                 not provided - method will allways modify first url.
         :param old_source_url (optional): for some SCM's jenkins allow set multiple repositories per job
                                 this parameter intended to indicate which repository need to be modified
@@ -325,18 +351,22 @@ class Job(JenkinsBase):
             self.update_config(ET.tostring(element_tree))
         else:
             for scm_url in scm_url_list:
-                if scm_url.text == old_source_url: 
+                if scm_url.text == old_source_url:
                     scm_url.text = new_source_url
                     self.update_config(ET.tostring(element_tree))
 
+    def get_config_xml_url(self):
+        return '%s/config.xml' % self.baseurl
+
     def update_config(self, config):
         """
         Update the config.xml to the job
         Also refresh the ElementTree object since the config has changed
         """
-        post_data = self.post_data("%(baseurl)s/config.xml" % self.__dict__, config)
+        url = self.get_config_xml_url()
+        response = self.jenkins.requester.post_url(url, params={}, data='')
         self._element_tree = ET.fromstring(config)
-        return post_data
+        return response.text
 
     def get_downstream_jobs(self):
         """
@@ -390,15 +420,19 @@ class Job(JenkinsBase):
             return []
         return upstream_jobs
 
+    def is_enabled(self):
+        self.poll()
+        return self._data["color"] != 'disabled'
+
     def disable(self):
         '''Disable job'''
-        disableurl = urlparse.urljoin(self.baseurl, 'disable' )
-        return self.post_data(disableurl, '')
+        url = "%s/disable" % self.baseurl
+        return self.get_jenkins_obj().requester.post_url(url, data='')
 
     def enable(self):
         '''Enable job'''
-        enableurl = urlparse.urljoin(self.baseurl, 'enable' )
-        return self.post_data(enableurl, '')
+        url = "%s/enable" % self.baseurl
+        return self.get_jenkins_obj().requester.post_url(url, data='')
 
     def delete_from_queue(self):
         """
@@ -408,14 +442,9 @@ class Job(JenkinsBase):
         if not self.is_queued():
             raise NotInQueue()
         queue_id = self._data['queueItem']['id']
-        cancelurl = urlparse.urljoin(self.get_jenkins_obj().get_queue().baseurl,
+        url = urlparse.urljoin(self.get_jenkins_obj().get_queue().baseurl,
                                      'cancelItem?id=%s' % queue_id)
-        try:
-            self.post_data(cancelurl, '')
-        except urllib2.HTTPError:
-            # The request doesn't have a response, so it returns 404,
-            # it's the expected behaviour
-            pass
+        self.get_jenkins_obj().requester.post_and_confirm_status(url, data='')
         return True
 
     def get_params(self):
